@@ -1,3 +1,4 @@
+# -*- coding: iso-8859-1 -*-
 """
 HTTP/1.1 client library
 
@@ -71,11 +72,8 @@ import errno
 import mimetools
 import socket
 from urlparse import urlsplit
-
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO
+from cStringIO import StringIO
+import cache.addrinfo
 
 __all__ = ["HTTP", "HTTPResponse", "HTTPConnection", "HTTPSConnection",
            "HTTPException", "NotConnected", "UnknownProtocol",
@@ -154,6 +152,9 @@ HTTP_VERSION_NOT_SUPPORTED = 505
 INSUFFICIENT_STORAGE = 507
 NOT_EXTENDED = 510
 
+# maximal amount of data to read at one time in _safe_read
+MAXAMOUNT = 1048576
+
 class HTTPMessage(mimetools.Message):
 
     def addheader(self, key, value):
@@ -270,7 +271,7 @@ class HTTPResponse:
     # See RFC 2616 sec 19.6 and RFC 1945 sec 6 for details.
 
     def __init__(self, sock, debuglevel=0, strict=0, method=None):
-        self.fp = sock.makefile('rb', 0)
+        self.fp = sock.makefile('rb')
         self.debuglevel = debuglevel
         self.strict = strict
         self._method = method
@@ -287,6 +288,10 @@ class HTTPResponse:
         self.length = _UNKNOWN          # number of bytes left in response
         self.will_close = _UNKNOWN      # conn will close at end of response
 
+    def __str__ (self):
+        return "<HTTPResponse status=%s reason=%s chunked=%s length=%s>" % \
+          (self.status, self.reason, self.chunked, self.length)
+
     def _read_status(self):
         # Initialize with Simple-Response defaults
         line = self.fp.readline()
@@ -295,7 +300,7 @@ class HTTPResponse:
         if not line:
             # Presumably, the server closed the connection before
             # sending a valid response.
-            raise BadStatusLine(line)
+            raise BadStatusLine("Bad status line %r" % line)
         try:
             [version, status, reason] = line.split(None, 2)
         except ValueError:
@@ -309,7 +314,7 @@ class HTTPResponse:
         if not version.startswith('HTTP/'):
             if self.strict:
                 self.close()
-                raise BadStatusLine(line)
+                raise BadStatusLine("Bad status line %r" % line)
             else:
                 # assume it's a Simple-Response from an 0.9 server
                 self.fp = LineAndFileWrapper(line, self.fp)
@@ -319,9 +324,9 @@ class HTTPResponse:
         try:
             status = int(status)
             if status < 100 or status > 999:
-                raise BadStatusLine(line)
+                raise BadStatusLine("Bad status line %r" % line)
         except ValueError:
-            raise BadStatusLine(line)
+            raise BadStatusLine("Bad status line %r" % line)
         return version, status, reason
 
     def begin(self):
@@ -354,6 +359,7 @@ class HTTPResponse:
             raise UnknownProtocol(version)
 
         if self.version == 9:
+            self.length = None
             self.chunked = 0
             self.will_close = 1
             self.msg = HTTPMessage(StringIO())
@@ -495,7 +501,7 @@ class HTTPResponse:
                     line = line[:i] # strip chunk-extensions
                 try:
                     chunk_left = int(line, 16)
-                except ValueError, msg:
+                except ValueError:
                     raise IncompleteRead("Invalid chunk length at %r" % line)
                 if chunk_left == 0:
                     break
@@ -544,24 +550,24 @@ class HTTPResponse:
         reading. If the bytes are truly not available (due to EOF), then the
         IncompleteRead exception can be used to detect the problem.
         """
-        s = ''
+        s = []
         while amt > 0:
-            chunk = self.fp.read(amt)
+            chunk = self.fp.read(min(amt, MAXAMOUNT))
             if not chunk:
                 raise IncompleteRead(s)
-            s += chunk
+            s.append(chunk)
             amt -= len(chunk)
-        return s
+        return ''.join(s)
 
     def getheader(self, name, default=None):
         if self.msg is None:
-            raise ResponseNotReady()
+            raise ResponseNotReady("Response msg is None")
         return self.msg.getheader(name, default)
 
     def getheaders(self):
         """Return list of (header, value) tuples."""
         if self.msg is None:
-            raise ResponseNotReady()
+            raise ResponseNotReady("Response msg is None")
         return self.msg.items()
 
 
@@ -587,6 +593,11 @@ class HTTPConnection:
         if strict is not None:
             self.strict = strict
 
+    def __str__ (self):
+        return "<HttpConnection state=%s\n method=%s\n" \
+               " sock=%s\n buffer=%s\n response=%s>" % \
+        (self.__state, self._method, self.sock, self._buffer, self.__response)
+
     def _set_hostport(self, host, port):
         if port is None:
             i = host.rfind(':')
@@ -610,8 +621,7 @@ class HTTPConnection:
     def connect(self):
         """Connect to the host and port specified in __init__."""
         msg = "getaddrinfo returns an empty list"
-        for res in socket.getaddrinfo(self.host, self.port, 0,
-                                      socket.SOCK_STREAM):
+        for res in cache.addrinfo.getaddrinfo(self.host, self.port):
             af, socktype, proto, canonname, sa = res
             try:
                 self.sock = socket.socket(af, socktype, proto)
@@ -621,9 +631,9 @@ class HTTPConnection:
             except socket.error, msg:
                 if self.debuglevel > 0:
                     print 'connect fail:', (self.host, self.port)
-                if self.sock:
+                if self.sock is not None:
                     self.sock.close()
-                self.sock = None
+                    self.sock = None
                 continue
             break
         if not self.sock:
@@ -645,7 +655,7 @@ class HTTPConnection:
             if self.auto_open:
                 self.connect()
             else:
-                raise NotConnected()
+                raise NotConnected("cannot send when not connected")
 
         # send the data to the server. if we get a broken pipe, then close
         # the socket. we want to reconnect when somebody tries to send again.
@@ -657,7 +667,7 @@ class HTTPConnection:
         try:
             self.sock.sendall(str)
         except socket.error, v:
-            if v[0] == 32:      # Broken pipe
+            if v[0] == errno.EPIPE:
                 self.close()
             raise
 
@@ -714,7 +724,7 @@ class HTTPConnection:
         if self.__state == _CS_IDLE:
             self.__state = _CS_REQ_STARTED
         else:
-            raise CannotSendRequest()
+            raise CannotSendRequest("cannot send request in state %s" % self.__state)
 
         # Save the method we use, we need it later in the response phase
         self._method = method
@@ -747,11 +757,20 @@ class HTTPConnection:
                     nil, netloc, nil, nil, nil = urlsplit(url)
 
                 if netloc:
-                    self.putheader('Host', netloc.encode("idna"))
-                elif self.port == HTTP_PORT:
-                    self.putheader('Host', self.host.encode("idna"))
+                    try:
+                        netloc_enc = netloc.encode("ascii")
+                    except UnicodeEncodeError:
+                        netloc_enc = netloc.encode("idna")
+                    self.putheader('Host', netloc_enc)
                 else:
-                    self.putheader('Host', "%s:%s" % (self.host.encode("idna"), self.port))
+                    try:
+                        host_enc = self.host.encode("ascii")
+                    except UnicodeEncodeError:
+                        host_enc = self.host.encode("idna")
+                    if self.port == HTTP_PORT:
+                        self.putheader('Host', host_enc)
+                    else:
+                        self.putheader('Host', "%s:%s" % (host_enc, self.port))
 
             # note: we are assuming that clients will not attempt to set these
             #       headers since *this* library must deal with the
@@ -782,7 +801,7 @@ class HTTPConnection:
         For example: h.putheader('Accept', 'text/html')
         """
         if self.__state != _CS_REQ_STARTED:
-            raise CannotSendHeader()
+            raise CannotSendHeader("cannot send request in state %s" % self.__state)
 
         str = '%s: %s' % (header, value)
         self._output(str)
@@ -793,7 +812,7 @@ class HTTPConnection:
         if self.__state == _CS_REQ_STARTED:
             self.__state = _CS_REQ_SENT
         else:
-            raise CannotSendHeader()
+            raise CannotSendHeader("cannot send request in state %s" % self.__state)
 
         self._send_output()
 
@@ -804,7 +823,7 @@ class HTTPConnection:
             self._send_request(method, url, body, headers)
         except socket.error, v:
             # trap 'Broken pipe' if we're allowed to automatically reconnect
-            if v[0] != 32 or not self.auto_open:
+            if v[0] != errno.EPIPE or not self.auto_open:
                 raise
             # try one more time
             self._send_request(method, url, body, headers)
@@ -853,7 +872,8 @@ class HTTPConnection:
         #                  isclosed() status to become true.
         #
         if self.__state != _CS_REQ_SENT or self.__response:
-            raise ResponseNotReady()
+            msg = "State %s, Reponse %s" % (self.__state, self.__response)
+            raise ResponseNotReady(msg)
 
         if self.debuglevel > 0:
             response = self.response_class(self.sock, self.debuglevel,
@@ -868,14 +888,17 @@ class HTTPConnection:
         self.__state = _CS_IDLE
 
         if response.will_close:
-            # this effectively passes the connection to the response
-            self.close()
+            # Pass the socket to the response
+            self.sock = None
         else:
             # remember this, so we can tell when it is complete
             self.__response = response
 
         return response
 
+    def is_idle (self):
+        return self.__state == _CS_IDLE
+
 # The next several classes are used to define FakeSocket,a socket-like
 # interface to an SSL connection.
 
@@ -1039,7 +1062,7 @@ class FakeSocket(SharedSocketClient):
 
     def makefile(self, mode, bufsize=None):
         if mode != 'r' and mode != 'rb':
-            raise UnimplementedFileMode()
+            raise UnimplementedFileMode("invalid file mode %r" % mode)
         return SSLFile(self._shared, self._ssl, bufsize)
 
     def send(self, stuff, flags = 0):
@@ -1141,7 +1164,7 @@ class HTTP:
 
             ### should we keep this behavior? do people use it?
             # keep the socket open (as a file), and return it
-            self.file = self._conn.sock.makefile('rb', 0)
+            self.file = self._conn.sock.makefile('rb')
 
             # close our socket -- we want to restart after any protocol error
             self.close()
@@ -1190,7 +1213,7 @@ if hasattr(socket, 'ssl'):
             self.cert_file = cert_file
 
 
-class HTTPException(Exception):
+class HTTPException(StandardError):
     # Subclasses that define an __init__ must call Exception.__init__
     # or define self.args.  Otherwise, str() will fail.
     pass
