@@ -291,9 +291,9 @@ class HtmlLogger (linkcheck.logger.Logger):
             self.writeln(u"<br>")
             self.stoptime = time.time()
             duration = self.stoptime - self.starttime
-            self.writeln(_("Stopped checking at %s (%s)") %
-                         (linkcheck.strformat.strtime(self.stoptime),
-                          linkcheck.strformat.strduration_long(duration)))
+            self.writeln(_("Stopped checking at %(time)s (%(duration)s)") %
+                 {"time": linkcheck.strformat.strtime(self.stoptime),
+                  "duration": linkcheck.strformat.strduration_long(duration)})
             self.writeln(u'</blockquote><br><hr noshade size="1"><small>'+
                          linkcheck.configuration.HtmlAppInfo+u"<br>")
             self.writeln(_("Get the newest version at %s") %
