@@ -79,13 +79,13 @@ class XMLLogger (linkcheck.logger.Logger):
         self.writeln(u'<?xml version="%s" encoding="%s"?>' %
                      (xmlquoteattr(version), xmlquoteattr(encoding)))
         if self.has_part("intro"):
-            self.comment(_("created by %s at %s") %
-                         (linkcheck.configuration.AppName,
-                          linkcheck.strformat.strtime(self.starttime)))
-            self.comment(_("Get the newest version at %s") %
-                         linkcheck.configuration.Url)
-            self.comment(_("Write comments and bugs to %s") %
-                         linkcheck.configuration.Email)
+            self.comment(_("created by %(app)s at %(time)s") %
+                        {"app": linkcheck.configuration.AppName,
+                         "time": linkcheck.strformat.strtime(self.starttime)})
+            self.comment(_("Get the newest version at %(url)s") %
+                         {'url': linkcheck.configuration.Url})
+            self.comment(_("Write comments and bugs to %(email)s") %
+                         {'email': linkcheck.configuration.Email})
             self.check_date()
             self.writeln()
 
@@ -96,9 +96,9 @@ class XMLLogger (linkcheck.logger.Logger):
         if self.has_part("outro"):
             self.stoptime = time.time()
             duration = self.stoptime - self.starttime
-            self.comment(_("Stopped checking at %s (%s)") %
-                         (linkcheck.strformat.strtime(self.stoptime),
-                          linkcheck.strformat.strduration_long(duration)))
+            self.comment(_("Stopped checking at %(time)s (%(duration)s)") %
+                 {"time": linkcheck.strformat.strtime(self.stoptime),
+                  "duration": linkcheck.strformat.strduration_long(duration)})
 
     def xml_starttag (self, name, attrs=None):
         """
