@@ -27,7 +27,6 @@ import socket
 import cStringIO as StringIO
 import Cookie
 
-import linkcheck
 import linkcheck.url
 import linkcheck.gzip2
 import linkcheck.strformat
@@ -37,6 +36,9 @@ import httpheaders as headers
 import internpaturl
 import proxysupport
 
+# helper alias
+unicode_safe = linkcheck.strformat.unicode_safe
+
 supportHttps = hasattr(linkcheck.httplib2, "HTTPSConnection") and \
                hasattr(socket, "ssl")
 
@@ -192,6 +194,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         # check response
         if response:
             self.check_response(response)
+            response.close()
 
     def check_http_connection (self):
         """
@@ -201,7 +204,10 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         @return: response or None if url is already handled
         @rtype: HttpResponse or None
         """
+        response = None
         while True:
+            if response is not None:
+                response.close()
             try:
                 response = self._get_http_response()
             except linkcheck.httplib2.BadStatusLine:
@@ -213,8 +219,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                     continue
                 raise
             if response.reason:
-                response.reason = \
-                        linkcheck.strformat.unicode_safe(response.reason)
+                response.reason = unicode_safe(response.reason)
             assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
                 "Response: %s %s", response.status, response.reason)
             assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
@@ -230,6 +235,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                          _("Enforced proxy %r ignored, aborting.") % newproxy,
                          valid=False)
                     return response
+                response.close()
                 response = self._get_http_response()
                 # restore old proxy settings
                 self.proxy, self.proxyauth = oldproxy
@@ -246,6 +252,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
             if tries == -1:
                 assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
                     "already handled")
+                response.close()
                 return None
             if tries >= self.max_redirects:
                 if self.method == "HEAD":
@@ -305,7 +312,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                          self.headers.getheader("Uri", ""))
             # make new url absolute and unicode
             newurl = urlparse.urljoin(redirected, newurl)
-            newurl = linkcheck.strformat.unicode_safe(newurl)
+            newurl = unicode_safe(newurl)
             assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
                 "Redirected to %r", newurl)
             self.add_info(_("Redirected to %(url)s.") % {'url': newurl},
@@ -374,6 +381,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                 # pretend to be finished and logged
                 return -1, response
             # new response data
+            response.close()
             response = self._get_http_response()
             tries += 1
         return tries, response
@@ -400,9 +408,8 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         else:
             if response.status == 204:
                 # no content
-                self.add_warning(
-                            linkcheck.strformat.unicode_safe(response.reason),
-                            tag="http-empty-content")
+                self.add_warning(unicode_safe(response.reason),
+                                 tag="http-empty-content")
             # store cookies for valid links
             if self.aggregate.config['storecookies']:
                 for c in self.cookies:
@@ -413,7 +420,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                                                      self.urlparts[1],
                                                      self.urlparts[2])
                     for h in out:
-                        self.add_info(linkcheck.strformat.unicode_safe(h))
+                        self.add_info(unicode_safe(h))
                 except Cookie.CookieError, msg:
                     self.add_warning(_("Could not store cookies: %(msg)s.") %
                                      {'msg': str(msg)},
@@ -482,7 +489,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                 self.url_connection.putheader(name, value)
         self.url_connection.endheaders()
         response = self.url_connection.getresponse()
-        self.persistent = headers.http_persistent(response)
+        self.persistent = not response.will_close
         self.timeout = headers.http_timeout(response)
         self.headers = response.msg
         # If possible, use official W3C HTTP response name
@@ -508,13 +515,14 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
             assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
                 "reuse cached HTTP(S) connection %s", conn)
             return conn
+        self.aggregate.connections.wait_for_host(host)
         if scheme == "http":
             h = linkcheck.httplib2.HTTPConnection(host)
         elif scheme == "https" and supportHttps:
             h = linkcheck.httplib2.HTTPSConnection(host)
         else:
-            raise linkcheck.LinkCheckerError(
-                                 _("Unsupported HTTP url scheme %r") % scheme)
+            msg = _("Unsupported HTTP url scheme %r") % scheme
+            raise linkcheck.LinkCheckerError(msg)
         if linkcheck.log.is_debug(linkcheck.LOG_CHECK):
             h.set_debuglevel(1)
         h.connect()
@@ -534,6 +542,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
             self.close_connection()
             t = time.time()
             response = self._get_http_response()
+            tries, response = self.follow_redirections(response)
             self.headers = response.msg
             self.data = response.read()
             encoding = headers.get_content_encoding(self.headers)
@@ -551,6 +560,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                     f = StringIO.StringIO(self.data)
                 self.data = f.read()
             self.downloadtime = time.time() - t
+            response.close()
         return self.data
 
     def is_html (self):
@@ -632,11 +642,10 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         # add to cached connections
         _user, _password = self.get_user_password()
         key = ("http", self.urlparts[1], _user, _password)
-        cache_add = self.aggregate.connections.add
-        # note: only cache the connection when it is persistent
-        # and all pending content has been received
-        if not self.persistent or not self.has_content or \
-           not cache_add(key, self.url_connection, self.timeout):
+        if self.persistent and self.url_connection.is_idle():
+            cache_add = self.aggregate.connections.add
+            cache_add(key, self.url_connection, self.timeout)
+        else:
             try:
                 self.url_connection.close()
             except:
