@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,12 @@ import gettext
 
 # more supported languages are added in init()
 supported_languages = ['en']
-default_language = None
+default_language = 'en'
+default_encoding = locale.getpreferredencoding()
+# It can happen that the preferrec encoding is not determinable, which
+# means the function returned None. Fall back to ASCII in this case.
+if default_encoding is None:
+    default_encoding = "ascii"
 
 def install_builtin (translator, do_unicode):
     """
@@ -56,8 +61,8 @@ class Translator (gettext.GNUTranslations):
 
 class NullTranslator (gettext.NullTranslations):
     """
-    A translation class always installing its gettext methods into the
-    default namespace.
+    A dummy translation class always installing its gettext methods into
+    the default namespace.
     """
 
     def install (self, do_unicode):
@@ -69,37 +74,35 @@ class NullTranslator (gettext.NullTranslations):
 
 def init (domain, directory):
     """
-    Initialize this gettext i18n module.
+    Initialize this gettext i18n module. Searches for supported languages
+    and installs the gettext translator class.
     """
     global default_language
-    # get supported languages
-    for lang in os.listdir(directory):
-        path = os.path.join(directory, lang)
-        if not os.path.isdir(path):
-            continue
-        if os.path.exists(os.path.join(path, 'LC_MESSAGES', '%s.mo'%domain)):
-            supported_languages.append(lang)
+    if os.path.isdir(directory):
+        # get supported languages
+        for lang in os.listdir(directory):
+            path = os.path.join(directory, lang, 'LC_MESSAGES')
+            if os.path.exists(os.path.join(path, '%s.mo' % domain)):
+                supported_languages.append(lang)
     loc = get_locale()
     if loc in supported_languages:
         default_language = loc
-    else:
-        default_language = "en"
     # install translation service routines into default namespace
-    translator = get_translator(domain, directory, fallback=True)
+    translator = get_translator(domain, directory,
+                                languages=[default_language], fallback=True)
     do_unicode = True
     translator.install(do_unicode)
 
 
 def get_translator (domain, directory, languages=None,
-                    translatorklass=Translator,
-                    fallback=False,
+                    translatorklass=Translator, fallback=False,
                     fallbackklass=NullTranslator):
     """
     Search the appropriate GNUTranslations class.
     """
     translator = gettext.translation(domain, localedir=directory,
             languages=languages, class_=translatorklass, fallback=fallback)
-    if not isinstance(translator, gettext.GNUTranslations):
+    if not isinstance(translator, gettext.GNUTranslations) and fallbackklass:
         translator = fallbackklass()
     return translator
 
@@ -120,8 +123,20 @@ def get_headers_lang (headers):
     if not headers.has_key('Accept-Language'):
         return default_language
     languages = headers['Accept-Language'].split(",")
-    # XXX sort with quality values
-    languages = [ lang.split(";")[0].strip() for lang in languages ]
+    # sort with preference values
+    pref_languages = []
+    for lang in languages:
+        pref = 1.0
+        if ";" in lang:
+            lang, _pref = lang.split(';', 1)
+            try:
+                pref = float(_pref)
+            except ValueError:
+                pass
+        pref_languages.append((pref, lang))
+    pref_languages.sort()
+    languages = [x[1] for x in pref_languages]
+    # search for lang
     for lang in languages:
         if lang in supported_languages:
             return lang
@@ -132,9 +147,15 @@ def get_locale ():
     """
     Return current configured locale.
     """
-    loc = locale.getdefaultlocale()[0]
+    loc = None
+    try:
+        loc = locale.getdefaultlocale()[0]
+    except ValueError:
+        # XXX ignore Python bug
+        # http://sourceforge.net/tracker/index.php?func=detail&aid=1158909&group_id=5470&atid=105470
+        pass
     if loc is None:
-        loc = 'C'
+        return 'C'
     loc = locale.normalize(loc)
     # split up the locale into its base components
     pos = loc.find('@')
