@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,17 +25,18 @@ import time
 import urlparse
 import types
 
-import linkcheck
 import linkcheck.configuration
 import linkcheck.url
 import linkcheck.i18n
 import linkcheck.strformat
+import linkcheck.checker
+import linkcheck.director
 
 _logfile = None
 _supported_langs = ('de', 'fr', 'nl', 'C')
-_is_level = re.compile(r'^[0123]$').match
+_is_level = re.compile(r'^(0|1|2|3|-1)$').match
 
-class FormError (Exception):
+class FormError (StandardError):
     """
     Form related errors.
     """
@@ -51,10 +52,14 @@ def startoutput (out=sys.stdout):
               "Pragma: no-cache\r\n"
               "\r\n")
 
-def checkaccess (out=sys.stdout, hosts=[], servers=[], env=os.environ):
+def checkaccess (out=sys.stdout, hosts=None, servers=None, env=os.environ):
     """
     See if remote addr is allowed to access the CGI interface.
     """
+    if hosts is None:
+        hosts = []
+    if servers is None:
+        servers = []
     if os.environ.get('REMOTE_ADDR') in hosts and \
        os.environ.get('SERVER_ADDR') in servers:
         return True
@@ -63,46 +68,43 @@ def checkaccess (out=sys.stdout, hosts=[], servers=[], env=os.environ):
     return False
 
 
-def checklink (out=sys.stdout, form={}, env=os.environ):
+def checklink (out=sys.stdout, form=None, env=os.environ):
     """
     Main cgi function, check the given links and print out the result.
     """
+    if form is None:
+        form = {}
     try:
         checkform(form)
     except FormError, why:
         logit(form, env)
         print_error(out, why)
         return
-    import linkcheck.configuration
     config = linkcheck.configuration.Configuration()
     config["recursionlevel"] = int(form["level"].value)
     config["logger"] = config.logger_new('html', fd=out)
     config["threads"] = 0
-    if form.has_key('externstrictall'):
-        config['externstrictall'] = True
     if form.has_key("anchors"):
         config["anchors"] = True
     if not form.has_key("errors"):
         config["verbose"] = True
-    if form.has_key("intern"):
-        pat = linkcheck.url.safe_host_pattern(re.escape(get_host_name(form)))
-    else:
-        pat = linkcheck.url.safe_url_pattern
-    config["internlinks"].append(linkcheck.get_link_pat("^%s$" % pat))
     # avoid checking of local files or other nasty stuff
-    config["externlinks"].append(
-             linkcheck.get_link_pat("^%s$" % linkcheck.url.safe_url_pattern))
-    config["externlinks"].append(linkcheck.get_link_pat(".*", strict=True))
-    import linkcheck.checker
-    import linkcheck.checker.cache
-    import linkcheck.checker.consumer
-    cache = linkcheck.checker.cache.Cache()
-    consumer = linkcheck.checker.consumer.Consumer(config, cache)
+    pat = "!^%s$" % linkcheck.url.safe_url_pattern
+    config["externlinks"].append(linkcheck.get_link_pat(pat, strict=True))
     # start checking
+    aggregate = linkcheck.director.get_aggregate(config)
+    get_url_from = linkcheck.checker.get_url_from
     url = form["url"].value
-    url_data = linkcheck.checker.get_url_from(url, 0, consumer)
-    consumer.append_url(url_data)
-    linkcheck.checker.check_urls(consumer)
+    url_data = get_url_from(url, 0, aggregate, assume_local=False)
+    try:
+        linkcheck.add_intern_pattern(url_data, config)
+    except UnicodeError:
+        logit({}, env)
+        print_error(out,
+                    "URL has unparsable domain name: %s" % sys.exc_info()[1])
+        return
+    aggregate.urlqueue.put(url_data)
+    linkcheck.director.check_urls(aggregate)
 
 
 def get_host_name (form):
@@ -141,7 +143,7 @@ def checkform (form):
         if not _is_level(level):
             raise FormError(_("invalid recursion level"))
     # check options
-    for option in ("externstrictall", "anchors", "errors", "intern"):
+    for option in ("anchors", "errors", "intern"):
         if form.has_key(option):
             if not form[option].value == "on":
                 raise FormError(_("invalid %s option syntax") % option)
@@ -181,4 +183,3 @@ Errors are logged.
 </blockquote>
 </body>
 </html>""") % why)
-
