@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@ import os
 import urlparse
 import urllib
 
+urlparse.uses_netloc.extend(('ldap', 'irc'))
+
 # constants defining url part indexes
 SCHEME = 0
 HOSTNAME = DOMAIN = 1
@@ -49,7 +51,7 @@ _safe_char = r"([%(_az09)s%(_path)s\+]|"\
 _safe_scheme_pattern = r"(https?|ftp)"
 _safe_domain_pattern = r"([%(_az09)s][%(_az09)s\-]*"\
                        r"(\.[%(_az09)s][%(_az09)s\-]*)*\.?)" % _basic
-_safe_host_pattern = _safe_domain_pattern+r"(:(80|8080|8000))?" % _basic
+_safe_host_pattern = _safe_domain_pattern+r"(:(80|8080|8000|443))?" % _basic
 _safe_path_pattern = r"((/([%(_az09)s%(_path)s]|"\
                      r"(%%[%(_hex_safe)s][%(_hex_full)s]))+)*/?)" % _basic
 _safe_fragment_pattern = r"%s*" % _safe_char
@@ -76,7 +78,7 @@ def splitparams (path):
     Split off parameter part from path.
     Returns tuple (path-without-param, param)
     """
-    if '/'  in path:
+    if '/' in path:
         i = path.find(';', path.rfind('/'))
     else:
         i = path.find(';')
@@ -85,36 +87,16 @@ def splitparams (path):
     return path[:i], path[i+1:]
 
 
-def is_safe_js_url (urlstr):
-    """
-    Test javascript URL strings.
-    """
-    url = list(urlparse.urlsplit(urlstr))
-    if url[0].lower() != 'http':
-        return False
-    if not is_safe_host(url[1]):
-        return False
-    if ";" in urlstr:
-        url[2], parameter = splitparams(url[2])
-        if not is_safe_parameter(parameter):
-            return False
-    if not is_safe_path(url[2]):
-        return False
-    if not is_safe_query(url[3]):
-        return False
-    if not is_safe_fragment(url[4]):
-        return False
-    return True
-
-
 def is_numeric_port (portstr):
     """
-    return True iff portstr is a valid port number
+    return: integer port (== True) iff portstr is a valid port number,
+           False otherwise
     """
     if portstr.isdigit():
         port = int(portstr)
         # 65536 == 2**16
-        return 0 < port < 65536
+        if 0 < port < 65536:
+            return port
     return False
 
 
@@ -151,16 +133,26 @@ def parse_qsl (qs, keep_blank_values=0, strict_parsing=0):
         false (the default), errors are silently ignored. If true,
         errors raise a ValueError exception.
     @type strict_parsing: bool
-    @returns: a list, as G-d intended.
-    @rtype: list
+    @returns: list of triples (key, value, separator) where key and value
+      are the splitted CGI parameter and separator the used separator
+      for this CGI parameter which is either a semicolon or an ampersand
+    @rtype: list of triples
     """
-    pairs = [s2 for s1 in qs.split('&') for s2 in s1.split(';')]
+    pairs = []
+    name_value_amp = qs.split('&')
+    for name_value in name_value_amp:
+        if ';' in name_value:
+            pairs.extend([[x, ';'] for x in name_value.split(';')])
+            pairs[-1][1] = '&'
+        else:
+            pairs.append([name_value, '&'])
+    pairs[-1][1] = ''
     r = []
-    for name_value in pairs:
+    for name_value, sep in pairs:
         nv = name_value.split('=', 1)
         if len(nv) != 2:
             if strict_parsing:
-                raise ValueError, "bad query field: %s" % `name_value`
+                raise ValueError("bad query field: %s" % `name_value`)
             elif len(nv) == 1:
                 # None value indicates missing equal sign
                 nv = (nv[0], None)
@@ -172,7 +164,7 @@ def parse_qsl (qs, keep_blank_values=0, strict_parsing=0):
                 value = urllib.unquote(nv[1].replace('+', ' '))
             else:
                 value = nv[1]
-            r.append((name, value))
+            r.append((name, value, sep))
     return r
 
 
@@ -180,6 +172,7 @@ def idna_encode (host):
     """
     Encode hostname as internationalized domain name (IDN) according
     to RFC 3490.
+    @raise: UnicodeError if hostname is not properly IDN encoded.
     """
     if host and isinstance(host, unicode):
         uhost = host.encode('idna').decode('ascii')
@@ -219,16 +212,15 @@ def url_fix_host (urlparts):
             userpass += "@"
         else:
             userpass = ""
-        host, port = urllib.splitnport(host)
-        # remove trailing dot
-        if host.endswith("."):
-            host = host[:-1]
-        # remove a default (or invalid) port
-        if port in [-1, None, default_ports.get(urlparts[0])]:
-            urlparts[1] = userpass+host
-        else:
-            urlparts[1] = "%s%s:%d" % (userpass, host, port)
-
+        if urlparts[0] in default_ports:
+            dport = default_ports[urlparts[0]]
+            host, port = splitport(host, port=dport)
+            if host.endswith("."):
+                host = host[:-1]
+            if port != dport:
+                host = "%s:%d" % (host, port)
+        urlparts[1] = userpass+host
+    return is_idn
 
 def url_fix_common_typos (url):
     """
@@ -253,24 +245,26 @@ def url_parse_query (query):
     """
     Parse and re-join the given CGI query.
     """
+    if isinstance(query, unicode):
+        query = query.encode('iso8859-1', 'ignore')
     # if ? is in the query, split it off, seen at msdn.microsoft.com
     if '?' in query:
         query, append = query.split('?', 1)
-        append = '?'+append
+        append = '?'+url_parse_query(append)
     else:
         append = ""
     l = []
-    for k, v in parse_qsl(query, True):
-        k = urllib.quote(k, '/-:,')
+    for k, v, sep in parse_qsl(query, True):
+        k = url_quote_part(k, '/-:,; ').replace(' ', '+')
         if v:
-            v = urllib.quote(v, '/-:,')
-            l.append("%s=%s" % (k, v))
+            v = url_quote_part(v, '-:,; ').replace(' ', '+')
+            l.append("%s=%s%s" % (k, v, sep))
         elif v is None:
-            l.append(k)
+            l.append("%s%s" % (k, sep))
         else:
             # some sites do not work when the equal sign is missing
-            l.append("%s=" % k)
-    return '&'.join(l) + append
+            l.append("%s=%s" % (k, sep))
+    return ''.join(l) + append
 
 
 def url_norm (url):
@@ -291,27 +285,36 @@ def url_norm (url):
     is_idn = url_fix_host(urlparts)
     # query
     urlparts[3] = url_parse_query(urlparts[3])
-    if not urlparts[2]:
-        # empty path should be a slash, but not in certain schemes
-        # note that in relative links, urlparts[0] might be empty
-        # in this case, do not make any assumptions
-        if urlparts[0] and urlparts[0] not in urlparse.non_hierarchical:
-            urlparts[2] = '/'
-    else:
-        # fix redundant path parts
-        urlparts[2] = collapse_segments(urlparts[2])
+    is_hierarchical = urlparts[0] not in urlparse.non_hierarchical
+    if is_hierarchical:
+        # URL has a hierarchical path we should norm
+        if not urlparts[2]:
+            # Empty path is allowed if both query and fragment are also empty.
+            # Note that in relative links, urlparts[0] might be empty.
+            # In this case, do not make any assumptions.
+            if urlparts[0] and (urlparts[3] or urlparts[4]):
+                urlparts[2] = '/'
+        else:
+            # fix redundant path parts
+            urlparts[2] = collapse_segments(urlparts[2])
+    # anchor
+    urlparts[4] = urllib.unquote(urlparts[4])
     # quote parts again
-    urlparts[0] = urllib.quote(urlparts[0]) # scheme
-    urlparts[1] = urllib.quote(urlparts[1], '@:') # host
-    urlparts[2] = urllib.quote(urlparts[2], _nopathquote_chars) # path
+    urlparts[0] = url_quote_part(urlparts[0]) # scheme
+    urlparts[1] = url_quote_part(urlparts[1], '@:') # host
+    urlparts[2] = url_quote_part(urlparts[2], _nopathquote_chars) # path
+    urlparts[4] = url_quote_part(urlparts[4]) # anchor
     res = urlparse.urlunsplit(urlparts)
     if url.endswith('#') and not urlparts[4]:
         # re-append trailing empty fragment
         res += '#'
+    if isinstance(url, unicode):
+        res = unicode(res)
     return (res, is_idn)
 
 
 _slashes_ro = re.compile(r"/+")
+_thisdir_ro = re.compile(r"^\./")
 _samedir_ro = re.compile(r"/\./|/\.$")
 _parentdir_ro = re.compile(r"^/(\.\./)+|/(?!\.\./)[^/]+/\.\.(/|$)")
 _relparentdir_ro = re.compile(r"^(?!\.\./)[^/]+/\.\.(/|$)")
@@ -325,13 +328,14 @@ def collapse_segments (path):
     # backslashes to be left alone, and finally quoted with '%5C')
     # But replacing has several positive effects:
     # - Prevents path attacks on Windows systems (using \.. parent refs)
-    # - Fixes bad urls where users used backslashes instead of slashes.
+    # - Fixes bad URLs where users used backslashes instead of slashes.
     #   This is a far more probable case than users having an intentional
     #   backslash in the path name.
     path = path.replace('\\', '/')
     # shrink multiple slashes to one slash
     path = _slashes_ro.sub("/", path)
     # collapse redundant path segments
+    path = _thisdir_ro.sub("", path)
     path = _samedir_ro.sub("/", path)
     # collapse parent path segments
     # note: here we exploit the fact that the replacements happen
@@ -359,29 +363,39 @@ def url_quote (url):
     if not url_is_absolute(url):
         return document_quote(url)
     urlparts = list(urlparse.urlsplit(url))
-    urlparts[0] = urllib.quote(urlparts[0]) # scheme
-    urlparts[1] = urllib.quote(urlparts[1], ':') # host
-    urlparts[2] = urllib.quote(urlparts[2], '/=,') # path
-    urlparts[3] = urllib.quote(urlparts[3], '&=,') # query
+    urlparts[0] = url_quote_part(urlparts[0]) # scheme
+    urlparts[1] = url_quote_part(urlparts[1], ':') # host
+    urlparts[2] = url_quote_part(urlparts[2], '/=,') # path
+    urlparts[3] = url_quote_part(urlparts[3], '&=,') # query
     l = []
-    for k, v in parse_qsl(urlparts[3], True): # query
-        k = urllib.quote(k, '/-:,')
+    for k, v, sep in parse_qsl(urlparts[3], True): # query
+        k = url_quote_part(k, '/-:,')
         if v:
-            v = urllib.quote(v, '/-:,')
-            l.append("%s=%s" % (k, v))
+            v = url_quote_part(v, '/-:,')
+            l.append("%s=%s%s" % (k, v, sep))
         else:
-            l.append(k)
-    urlparts[3] = '&'.join(l)
-    urlparts[4] = urllib.quote(urlparts[4]) # anchor
+            l.append("%s%s" % (k, sep))
+    urlparts[3] = ''.join(l)
+    urlparts[4] = url_quote_part(urlparts[4]) # anchor
     return urlparse.urlunsplit(urlparts)
 
 
+def url_quote_part (s, safechars='/'):
+    """
+    Wrap urllib.quote() to support unicode strings. A unicode string
+    is first converted to ISO-8859-1, invalid characters are ignored.
+    After that urllib.quote() is called.
+    """
+    if isinstance(s, unicode):
+        s = s.encode("iso-8859-1", "ignore")
+    return urllib.quote(s, safechars)
+
 def document_quote (document):
     """
     Quote given document.
     """
     doc, query = urllib.splitquery(document)
-    doc = urllib.quote(doc, '/=,')
+    doc = url_quote_part(doc, '/=,')
     if query:
         return "%s?%s" % (doc, query)
     return doc
@@ -411,10 +425,10 @@ def match_host (host, domainlist):
     return False
 
 
-_nopathquote_chars = r"-;/=,~*+()@"
+_nopathquote_chars = "-;/=,~*+()@!"
 if os.name == 'nt':
     _nopathquote_chars += "|"
-_safe_url_chars = _nopathquote_chars + r"a-zA-Z0-9_:\.&#%\?"
+_safe_url_chars = re.escape(_nopathquote_chars + "_:.&#%?[]!")+"a-zA-Z0-9"
 _safe_url_chars_ro = re.compile(r"^[%s]*$" % _safe_url_chars)
 def url_needs_quoting (url):
     """
@@ -441,12 +455,29 @@ def url_split (url):
     port = default_ports.get(scheme, 80)
     if host:
         host = host.lower()
-        host, port = urllib.splitnport(host, port)
+        host, port = splitport(host, port=port)
     return scheme, host, port, document
 
 
-def url_unicode_split (url):
+def url_unsplit (parts):
+    if parts[2] == default_ports.get(parts[0]):
+        return "%s://%s%s" % (parts[0], parts[1], parts[3])
+    return "%s://%s:%d%s" % parts
+
+
+def splitport (host, port=80):
     """
-    Like urlparse.urlsplit(), but always returning unicode parts.
+    Split optional port number from host. If host has no port number,
+    the given default port is returned.
+
+    @param host: host name
+    @ptype host: string
+    @return: tuple of (host, port)
+    @rtype: tuple of (string, int)
     """
-    return [unicode(s) for s in urlparse.urlsplit(url)]
+    if ":" in host:
+        host, sport = host.split(":", 1)
+        iport = is_numeric_port(sport)
+        if iport:
+            port = iport
+    return host, port
